@@ -19,7 +19,6 @@ use ibc::{
             version::Version,
         },
         ics04_channel::{
-            self,
             channel::{ChannelEnd, IdentifiedChannelEnd},
             packet::{PacketMsgType, Sequence},
         },
@@ -45,7 +44,6 @@ use ibc_proto::ibc::core::{
 };
 
 use crate::{
-    chain::handle::requests::AppVersion,
     chain::StatusResponse,
     config::ChainConfig,
     connection::ConnectionMsgType,
@@ -272,14 +270,14 @@ where
                             self.get_key(reply_to)?
                         }
 
-                        Ok(ChainRequest::AppVersion { request, reply_to }) => {
-                            self.app_version(request, reply_to)?
-                        }
-
                         Ok(ChainRequest::AddKey { key_name, key, reply_to }) => {
                             self.add_key(key_name, key, reply_to)?
                         }
 
+                        Ok(ChainRequest::IbcVersion { reply_to }) => {
+                            self.ibc_version(reply_to)?
+                        }
+
 
                         Ok(ChainRequest::BuildHeader { trusted_height, target_height, client_state, reply_to }) => {
                             self.build_header(trusted_height, target_height, client_state, reply_to)?
@@ -483,15 +481,6 @@ where
         reply_to.send(result).map_err(Error::send)
     }
 
-    fn app_version(
-        &self,
-        request: AppVersion,
-        reply_to: ReplyTo<ics04_channel::Version>,
-    ) -> Result<(), Error> {
-        let result = self.chain.query_app_version(request);
-        reply_to.send(result).map_err(Error::send)
-    }
-
     fn add_key(
         &mut self,
         key_name: String,
@@ -502,6 +491,11 @@ where
         reply_to.send(result).map_err(Error::send)
     }
 
+    fn ibc_version(&mut self, reply_to: ReplyTo<Option<semver::Version>>) -> Result<(), Error> {
+        let result = self.chain.ibc_version();
+        reply_to.send(result).map_err(Error::send)
+    }
+
     fn build_header(
         &mut self,
         trusted_height: Height,
