@@ -1,14 +1,12 @@
 //! Relayer configuration
 
-mod error;
-mod proof_specs;
-pub mod reload;
+pub mod error;
+pub mod filter;
+pub mod proof_specs;
 pub mod types;
 
 use alloc::collections::BTreeMap;
-use alloc::collections::BTreeSet;
 use core::{fmt, time::Duration};
-use itertools::Itertools;
 use std::sync::{Arc, RwLock};
 use std::{fs, fs::File, io::Write, path::Path};
 
@@ -24,6 +22,8 @@ use crate::keyring::Store;
 
 pub use error::Error;
 
+pub use filter::PacketFilter;
+
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 pub struct GasPrice {
     pub price: f64,
@@ -42,64 +42,6 @@ impl fmt::Display for GasPrice {
     }
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
-#[serde(
-    rename_all = "lowercase",
-    tag = "policy",
-    content = "list",
-    deny_unknown_fields
-)]
-pub enum PacketFilter {
-    Allow(ChannelsSpec),
-    Deny(ChannelsSpec),
-    AllowAll,
-}
-
-impl Default for PacketFilter {
-    /// By default, allows all channels & ports.
-    fn default() -> Self {
-        Self::AllowAll
-    }
-}
-
-impl PacketFilter {
-    /// Returns true if the packets can be relayed on the channel with [`PortId`] and [`ChannelId`],
-    /// false otherwise.
-    pub fn is_allowed(&self, port_id: &PortId, channel_id: &ChannelId) -> bool {
-        match self {
-            PacketFilter::Allow(spec) => spec.contains(&(port_id.clone(), channel_id.clone())),
-            PacketFilter::Deny(spec) => !spec.contains(&(port_id.clone(), channel_id.clone())),
-            PacketFilter::AllowAll => true,
-        }
-    }
-}
-
-#[derive(Clone, Debug, Default, Serialize, Deserialize)]
-#[serde(deny_unknown_fields)]
-pub struct ChannelsSpec(BTreeSet<(PortId, ChannelId)>);
-
-impl ChannelsSpec {
-    pub fn contains(&self, channel_port: &(PortId, ChannelId)) -> bool {
-        self.0.contains(channel_port)
-    }
-
-    pub fn iter(&self) -> impl Iterator<Item = &(PortId, ChannelId)> {
-        self.0.iter()
-    }
-}
-
-impl fmt::Display for ChannelsSpec {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "{}",
-            self.iter()
-                .map(|(pid, cid)| format!("{}/{}", pid, cid))
-                .join(", ")
-        )
-    }
-}
-
 /// Defaults for various fields
 pub mod default {
     use super::*;
@@ -465,9 +407,9 @@ mod tests {
             "/tests/config/fixtures/relayer_conf_example.toml"
         );
 
-        let config = load(path);
-        println!("{:?}", config);
-        assert!(config.is_ok());
+        let config = load(path).expect("could not parse config");
+
+        dbg!(config);
     }
 
     #[test]
