@@ -134,8 +134,8 @@ impl Packet {
 /// An object determines the amount of parallelism that can
 /// be exercised when processing [`IbcEvent`](ibc::events::IbcEvent)
 /// between two chains. For each [`Object`], a corresponding
-/// [`Worker`](crate::worker::Worker) is spawned and all [`IbcEvent`](ibc::events::IbcEvent)s mapped
-/// to an [`Object`] are sent to the associated [`Worker`](crate::worker::Worker)
+/// [`WorkerHandle`](crate::worker::WorkerHandle) is spawned and all [`IbcEvent`](ibc::events::IbcEvent)s mapped
+/// to an [`Object`] are sent to the associated [`WorkerHandle`](crate::worker::WorkerHandle)
 /// for processing.
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize)]
 #[serde(tag = "type")]
@@ -187,7 +187,7 @@ define_error! {
 }
 
 impl Object {
-    /// Returns `true` if this [`Object`] is for a [`Worker`](crate::worker::Worker)
+    /// Returns `true` if this [`Object`] is for a [`WorkerHandle`](crate::worker::WorkerHandle)
     /// which is interested in new block events originating from the chain with
     /// the given [`ChainId`]. Returns `false` otherwise.
     pub fn notify_new_block(&self, src_chain_id: &ChainId) -> bool {
@@ -381,29 +381,6 @@ impl Object {
         .into())
     }
 
-    /// Build the Packet object associated with the given
-    /// [`Open`](ibc::core::ics04_channel::channel::State::Open) channel event.
-    pub fn packet_from_chan_open_events(
-        attributes: &Attributes,
-        src_chain: &impl ChainHandle,
-    ) -> Result<Self, ObjectError> {
-        let channel_id = attributes
-            .channel_id()
-            .ok_or_else(|| ObjectError::missing_channel_id(attributes.clone()))?;
-
-        let dst_chain_id =
-            counterparty_chain_from_channel(src_chain, channel_id, attributes.port_id())
-                .map_err(ObjectError::supervisor)?;
-
-        Ok(Packet {
-            dst_chain_id,
-            src_chain_id: src_chain.id(),
-            src_channel_id: channel_id.clone(),
-            src_port_id: attributes.port_id().clone(),
-        }
-        .into())
-    }
-
     /// Build the object associated with the given [`SendPacket`] event.
     pub fn for_send_packet(
         e: &SendPacket,
