@@ -1,12 +1,11 @@
 mod block_on;
 pub use block_on::block_on;
 
-mod recv_multiple;
-pub use recv_multiple::try_recv_multiple;
-
 pub mod bigint;
 pub mod diff;
 pub mod iter;
+pub mod lock;
 pub mod queue;
 pub mod retry;
 pub mod stream;
+pub mod task;
